suppressPackageStartupMessages({
  library(Seurat)
  library(tidyverse)
  library(harmony)
  library(patchwork)
  library(ggforce)
})

theme_set(theme_bw())

#----GEX----
gex <- read_rds(here::here("data", "gex.rds")) #The concatenated GEX matrices

gex <- CreateSeuratObject(gex)

gex$patient <- str_extract(colnames(gex), "Pt[0-9]{1,3}")
gex$antigen <- str_extract(colnames(gex), "(?<=Pt[0-9]{1,3}[A-Z]{0,1}_)[:alnum:]*")
gex$sample <- paste(gex$patient, gex$antigen, sep = "_")

gex <- gex |>
  NormalizeData() |>
  FindVariableFeatures() |>
  ScaleData(split.by = "sample") |>
  RunPCA(verbose = FALSE) |>
  RunHarmony(group.by.vars = "patient", dims.use = 1:5) |>
  FindNeighbors(reduction = "harmony") |>
  FindClusters(resolution = 0.15) |>
  RunUMAP(reduction = "harmony", dims = 1:5)

activation_genes <- c("IFNG", "TNF", "CD69", "TNFRSF9", "PDCD1")

#----UMAP Plots----
plotlist <- UMAPPlot(gex, combine = FALSE)
plotlist[[1]] <- plotlist[[1]] +
  ggtitle("clusters")

plotlist <- c(plotlist, FeaturePlot(gex, activation_genes, combine = FALSE))

wrap_plots(plotlist, ncol = 3)

#----Facet matrix----
scale_ident_to_expression <- function(ident, ex, levels = length(activation_genes), invert = TRUE){ #Helper function for y scaling in ridge plots of facet matrix
  grp <- rep(1:levels, each = length(ex) / levels)
  ex <- split(ex, grp)
  ident <- as.integer(ident)
  
  if (invert) {
    ident <- abs(ident - max(ident))
  } else {
    ident <- ident - 1
  }
  lvls <- split(ident, grp)
  step <- lapply(ex, \(x) max(x) / (levels + 2))

  out <- map2(step, lvls, `*`) 
  
  unlist(out, use.names = FALSE)
}

activation_plot_data <- FetchData(gex, vars = c(activation_genes, "ident"))


activation_plot <- ggplot(activation_plot_data, aes(color = ident, fill = ident)) +
  geom_blank() +
  ggridges::geom_density_ridges2(mapping =
                                  aes(x = .panel_x,
                                      y = scale_ident_to_expression(ident, .panel_y),
                                      scale = 10,
                                      fill = ident),
                                inherit.aes = FALSE, show.legend = FALSE) +
  geom_autopoint(size = 0.5) +
  facet_matrix(vars(!ident), layer.diag = 2, layer.lower = 3, switch = "both") +
  theme(panel.grid = element_blank(), 
        panel.border = element_blank()) +
  labs(color = "clusters", fill = "clusters") +
  guides(color = guide_legend(override.aes = list(size = 5)))

activation_plot

gex$labels <- ifelse(gex$RNA_snn_res.0.15 == "2", "activated", "not activated")

#----Receptor data----
activated <- gex[[]] |>
  filter(labels == "activated", patient == "Pt038") |>
  rownames_to_column("barcode") |>
  mutate(barcode = str_extract(barcode, "[CGAT]{16}-[0-9]")) |>
  pull(barcode)

vdj_file <- here::here("data", "filtered_contig_annotations.csv")

tenX <- read_csv(vdj_file, show_col_types = FALSE) |>
  filter(barcode %in% activated, chain == "TRB") |>
  mutate(CDR3b = cdr3,
         TRBV = str_extract(v_gene, "TRBV[0-9-]{1,5}"),
         TRBJ = str_extract(j_gene, "TRBJ[0-9-]{1,5}"),
         CDR3a = NA,
         `subject:condition` = str_glue("38:single")) |>
  select(CDR3b:`subject:condition`) %>%
  group_by_all() |>
  summarise(count = n(), .groups = "drop")

bulk <- read_tsv(here::here("data", "bulk.tsv.zip"), show_col_types = FALSE) |>
  filter(str_detect(sample_name, "38"),
         str_detect(sample_name, regex("tu|mpre|pt", ignore_case = TRUE))) |>
  select(sample_name, amino_acid, v_resolved, j_resolved, templates) |>
  mutate(CDR3b = amino_acid,
         TRBV = str_replace(str_extract(v_resolved, "TCRBV[0-9-]{1,5}"), "TCR", "TR"), #to achieve uniform gene names
         TRBJ = str_replace(str_extract(j_resolved, "TCRBJ[0-9-]{1,5}"), "TCR", "TR"),
         CDR3a = NA,
         `subject:condition` = str_glue("38:tumor"),
         count = templates) |>
  select(CDR3b, TRBV, TRBJ, CDR3a, `subject:condition`, count) |>
  group_by_all() |>
  summarise(count = sum(count), .groups = "drop")

out38 <- here::here("output", "38.txt")

write_tsv(bind_rows(tenX, bulk), out38)

#----GLIPH2----
prefix <- here::here("gliph2/")
params <- str_glue("out_prefix=38
cdr3_file={out38}
refer_file={prefix}ref_CD8_v2.0.txt
v_usage_freq_file={prefix}ref_V_CD8_v2.0.txt
cdr3_length_freq_file={prefix}ref_L_CD8_v2.0.txt
local_min_pvalue=0.001
p_depth = 1000
global_convergence_cutoff = 1
simulation_depth=1000
kmer_min_depth=3
local_min_OVE=10
algorithm=GLIPH2
all_aa_interchangeable=1") # values taken from here: http://50.255.35.37:8080/tools

write_lines(params, here::here("gliph2", "38_parameter.txt"))

system2("gliph2/irtools.osx", args = str_glue("-c {here::here('gliph2', '38_parameter.txt')}")) #Calls GLIPH2 downloaded from http://50.255.35.37:8080

fs::file_move(list.files(pattern = "38"), here::here("output", list.files(pattern = "38")))

gliph <- read_csv(here::here("output", "38_cluster.csv"), show_col_types = F)

gliph_plot_data <- gliph |>
  group_by(type, Sample) |>
  summarise(freq = sum(Freq), .groups = "drop") |>
  mutate(Sample = str_extract(Sample, "(?<=:).*")) |>
  pivot_wider(names_from = Sample, values_from = freq) |>
  replace_na(replace = list(tumor = 0, single = 0)) |>
  select(type, tumor, single)

ggplot(gliph_plot_data, aes(x = single, y = tumor)) +
  geom_jitter(width = 0) +
  labs(x = "Activated Single Cells", y = "Bulk TCR beta Sequences from Tumor") +
  ggpp::geom_quadrant_lines(xintercept = .5, yintercept = .5) +
  ggpp::stat_quadrant_counts(xintercept = .1, yintercept = .1, quadrants = c(1,2,4), geom = "label", position = position_nudge(x = 0.15)) +
  scale_x_continuous(trans = scales::pseudo_log_trans(base = 10), breaks = c(0,1,2,5,10,20,50,100,200,300)) +
  scale_y_continuous(trans = scales::pseudo_log_trans(base = 10), breaks = c(0,1,2,5,10,20,50,100,200,400))
  theme(panel.grid.minor = element_blank())
